#!/usr/bin/env python
# -*- encoding: utf-8 -*-

"""
@File        :   eval_event_detector 
@Time        :   2023/9/26 21:22
@Author      :   Xuesong Chen
@Description :   
"""

import pandas as pd
from wuji.Evaluator.EventDetector import EventDetectorEvaluator

if __name__ == '__main__':
    # write a test case
    gt_df = pd.DataFrame({
        'Type': ['Apnea', 'Apnea', 'Hypopnea'],
        'Start': [20, 100, 200],
        'Duration': [20, 20, 20],
    })
    pred_df = pd.DataFrame({
        'Type': ['Apnea', 'Apnea', 'Apnea', 'Hypopnea'],
        'Start': [0, 20, 100, 200],
        'Duration': [5, 12, 20, 20],
    })
    evaluator = EventDetectorEvaluator(gt_df, pred_df)
    print(gt_df, pred_df)
    for average in ['macro', 'micro']:
        for label in ['Apnea', 'Hypopnea', 'overall']:
            print(average, label)
            print('f1:', evaluator.f1_score(label))
            print('precision:', evaluator.precision(label))
            print('recall:', evaluator.recall(label))



#!/usr/bin/env python
# -*- encoding: utf-8 -*-

"""
@File        :   HSP 
@Time        :   2023/12/13 16:42
@Author      :   Xuesong Chen
@Description :   
"""

import numpy as np
from datetime import datetime
import h5py
from wuji.Reader.EDF.Base import Base


class HSPEDFReader(Base):
    def __init__(self, file_path):
        super().__init__(file_path)

    def _parse_file(self, file_path):
        self.data = h5py.File(file_path, 'r')
        self.reader = self.data
        self.signal_labels = self.get_signal_labels()
        self._assign_signal_types()

    def get_signal_labels(self):
        label = self.data['hdr']['signal_labels'][:]
        signal_labels = []
        for i in range(label.shape[0]):
            channel_info = ''.join(chr(j[0]) for j in self.data[label[i][0]])
            signal_labels.append(channel_info)

        signal_labels = np.array(signal_labels)
        return signal_labels

    def _get_header_property(self, property_name):
        if property_name not in self.data['hdr']:
            return None
        property_data = self.data['hdr'][property_name]
        res = {}
        for sig_name, p in zip(self.signal_labels, property_data):
            res[sig_name] = np.squeeze(self.data[p[0]]).item()
        return res

    def get_signal_header(self):
        res = {}
        for _property in ['physical_min', 'physical_max', 'digital_min', 'digital_max']:
            _property_data = self._get_header_property(_property)
            res.update({_property: _property_data})
        return res

    def get_number_of_signals(self, type='ecg'):
        return np.sum(self.signal_type == type)

    def get_signal(self, ch_name=None, type='ecg', tmin=None, tmax=None, order=0):
        sampling_freq = self.get_sample_frequency()
        if ch_name:
            idx = np.argwhere(self.signal_labels == ch_name).flatten()
        else:
            if type not in self.signal_type:
                return None
            idx = np.argwhere(self.signal_type == type).flatten()[order]
        if tmin is None or tmax is None:
            return self.data['s'][:, idx]
        start_samp_idx = sampling_freq * tmin
        end_samp_idx = sampling_freq * tmax
        return self.data['s'][:, idx][start_samp_idx:end_samp_idx]

    def get_sample_frequency(self, type=None, order=0):
        return int(np.squeeze(self.data['recording']['samplingrate']))

    def get_start_recording_time(self):
        year, month, day = self.data['recording']['year'], self.data['recording']['month'], self.data['recording'][
            'day']
        hour, minute = self.data['recording']['hour'], self.data['recording']['minute']
        second, millisecond = self.data['recording']['second'], self.data['recording']['millisecond']
        get_single_value = lambda x: np.squeeze(x).astype(int).item()
        start_time = datetime(
            get_single_value(year),
            get_single_value(month),
            get_single_value(day),
            get_single_value(hour),
            get_single_value(minute),
            get_single_value(second),
            get_single_value(millisecond) * 1000,
        )
        return start_time

    def get_durations(self):
        sampling_freq = self.get_sample_frequency()
        return np.squeeze(self.data['samplesCount']) / sampling_freq


if __name__ == '__main__':
    from pprint import pprint as print

    # fp = r'E:\WJ\Shifted_Signal_20230723_212727268.mat'
    fp = r'/Shifted_Signal_000103d6a6d58c79552c49394b3adec2d300352b798aa764bde981b189b2e12e_20200412_205639268.mat'
    # fp = r'E:\WJ\HSP_data\Shifted_Signal_20110417_215226000.mat'
    file_name = '0010ee03836d2d4a4bac885d2f6fcb55c1c437e0ba5b6cf874f15d9d52d29ba7_20110417_215226000'
    hsp = HSPEDFReader(
        f'/Users/cxs/Downloads/hsp/Shifted_Signal_{file_name}.mat')
    # print(hsp.get_signal(type='nasal_pressure'))
    print(hsp.get_signal(type='ecg'))
    print(hsp.get_signal_labels())
    print(hsp.get_sample_frequency())
    print(hsp.get_start_recording_time())
    print(hsp.get_durations())
    print(hsp.get_number_of_signals(type='flow'))
    print(hsp.get_signal_header())



#!/usr/bin/env python
"""Top-level module for sed_eval"""

# Import all submodules
from . import sound_event
from . import scene
from . import audio_tag
from . import io
from . import util
from . import metric
from . import test

__version__ = '0.2.1'


#!/usr/bin/env python
# -*- encoding: utf-8 -*-

"""
@File        :   AnnotaitonReader 
@Time        :   2023/8/3 14:12
@Author      :   Xuesong Chen
@Description :   
"""
import re
from datetime import datetime, timedelta

import pandas as pd

from wuji.Reader.Annotation.Base import Base, AHEventFilter


class HSPHumanInfoReader:
    def __init__(self, path):
        self.df = pd.read_csv(path)
        self.df['HashFolderName'] = self.df['HashFolderName'].apply(lambda x: x.strip())
        self.df.set_index('HashFolderName', inplace=True)

    def get_DateOfVisit(self, hash_folder_name=''):
        datetime_str = self.df.loc[hash_folder_name, 'ShiftedCreationTime'][:26]
        date_object = datetime.strptime(datetime_str, '%Y-%m-%d %H:%M:%S.%f').date()
        return date_object


def convert_to_relative_time(time_str, start_time):
    time_obj = datetime.strptime(time_str, '%H:%M:%S')
    time_obj = datetime(
        start_time.year, start_time.month, start_time.day,
        time_obj.hour, time_obj.minute, time_obj.second
    )
    relative_time_delta = time_obj - start_time
    # Handle the case where the time passes midnight
    if relative_time_delta.total_seconds() < 0:
        relative_time_delta += timedelta(days=1)
    return relative_time_delta.total_seconds()


class HSPAnnotationReader(Base):
    def __init__(self, file_path, start_date=None):
        self.respiratory_events_df = None
        self.start_date = start_date
        self.AH_events = None
        self.arousal_events = None
        self.OD_events = None
        super().__init__(file_path)

    def _parse_file(self, file_path):
        self.df = pd.read_csv(file_path, header=0, index_col=0)
        start_time_str = self.df.iloc[0]['time'].strip()
        if '.' in start_time_str:
            start_time = datetime.strptime(start_time_str, '%H:%M:%S.%f')
        else:
            start_time = datetime.strptime(start_time_str, '%H:%M:%S')
        self.recording_start_time = datetime(
            self.start_date.year, self.start_date.month, self.start_date.day,
            start_time.hour, start_time.minute, start_time.second
        )
        self.df.dropna(inplace=True)
        self.df['Start'] = self.df['time'].apply(lambda x: convert_to_relative_time(x, self.recording_start_time))
        self.df.rename(columns={'duration': 'Duration', 'event': 'Type'}, inplace=True)

    def get_standard_sleep_stages(self, drop_not_scored=False):
        stage_abbr_dic = {
            'Sleep_stage_W': 'Wake',
            'Sleep_stage_N1': 'N1',
            'Sleep_stage_N2': 'N2',
            'Sleep_stage_N3': 'N3',
            'Sleep_stage_R': 'REM',
            'Sleep_stage_REM': 'REM',
        }
        if not drop_not_scored:
            stage_abbr_dic['Sleep_stage_?'] = 'NotScored'
        assert len(self.df['Type'][self.df['Type'].str.contains('Sleep_stage')].unique()) <= 6, print(
            f"Sleep stage error: {self.df['Type'][self.df['Type'].str.contains('Sleep_stage')].unique()}")
        patterns = '|'.join(stage_abbr_dic.keys())
        self.standard_sleep_stages = self.df[self.df['Type'].str.contains(patterns)].replace(stage_abbr_dic)[
            ['Type', 'Start', 'Duration']]
        self.standard_sleep_stages.reset_index(drop=True, inplace=True)
        return self.standard_sleep_stages

    def get_respiratory_events(self):
        respiratory_events_df = self.df[self.df['Type'].str.contains('Apnea|Hypopnea')].copy()

        def map_to_specific_event(event):
            if 'Hypopnea' in event:
                return 'Hypopnea'
            else:
                return 'Apnea'

        respiratory_events_df['Type'] = respiratory_events_df['Type'].apply(map_to_specific_event)
        assert len(respiratory_events_df['Type'].unique()) <= 2, print(
            f"Respiratory event error: {respiratory_events_df['Type'].unique()}")
        self.respiratory_events = respiratory_events_df[['Type', 'Start', 'Duration']][
            (respiratory_events_df['Duration'] >= 10) & (respiratory_events_df['Duration'] <= 120)]
        self.respiratory_events.reset_index(drop=True, inplace=True)
        return self.respiratory_events

    def get_standard_AH_events(self, type='AHI3', od_eps=45, aro_eps=6):
        if self.standard_sleep_stages is None:
            self.get_standard_sleep_stages()
        if self.OD_events is None:
            self.get_OD_events()
        if self.respiratory_events is None:
            self.get_respiratory_events()
        if self.arousal_events is None:
            self.get_arousal_events()
        filter = AHEventFilter(self.respiratory_events, self.OD_events, self.arousal_events, self.standard_sleep_stages)
        self.OD_events = self.OD_events[['Type', 'Start', 'Duration']]
        self.arousal_events = self.arousal_events[['Type', 'Start', 'Duration']]
        self.AH_events = filter.get_filtered_AH_events(type=type, od_eps=od_eps, aro_eps=aro_eps)
        return self.AH_events

    def get_arousal_events(self):
        self.arousal_events = self.df[self.df['Type'].str.contains('Arousal|RERA')][['Type', 'Start', 'Duration']]
        self.arousal_events.reset_index(drop=True, inplace=True)
        return self.arousal_events

    def get_REAR(self):
        self.rear = self.df[self.df['Type'].str.contains('RERA')][['Type', 'Start', 'Duration']]
        self.rear.reset_index(drop=True, inplace=True)
        return self.rear

    def get_OD_events(self):
        self.OD_events = self.df[self.df['Type'].str.contains('Desaturation')][['Type', 'Start', 'Duration']]
        self.OD_events.reset_index(drop=True, inplace=True)
        return self.OD_events


if __name__ == '__main__':
    hash_name = '0010ee03836d2d4a4bac885d2f6fcb55c1c437e0ba5b6cf874f15d9d52d29ba7_20110417_215226000'
    human_info_path = '/Users/cxs/Downloads/hsp/bdsp_opendata_Sleep_bdsp_psg_master_20221110.csv'
    human_info_reader = HSPHumanInfoReader(human_info_path)
    date = human_info_reader.get_DateOfVisit(hash_name)
    reader = HSPAnnotationReader(
        f'/Users/cxs/Downloads/hsp/{hash_name}_annotations.csv', start_date=date
    )
    print('reader', reader.get_recording_start_time())
    print(reader.get_standard_sleep_stages())
    print(reader.get_respiratory_events())
    # print(len(reader.get_REAR()), reader.get_REAR())
    print(len(reader.get_arousal_events()), reader.get_arousal_events())
    print(len(reader.get_OD_events()), reader.get_OD_events())
    print(len(reader.get_standard_AH_events()), reader.get_standard_AH_events(), sep='\n')
    print()



#!/usr/bin/env python
# -*- encoding: utf-8 -*-

"""
@File        :   read_edf_of_Philips 
@Time        :   2023/9/21 10:26
@Author      :   Xuesong Chen
@Description :   
"""
import pandas as pd
from matplotlib import pyplot as plt

from wuji.Reader.EDF.Philips import PhilipsEDFReader
from wuji.Reader.Annotation.Philips import PhilipsAnnotationReader

edf_file_path = '/Users/cxs/Downloads/scored studies/00000818-113072/00000818-113072_2949177.edf'
annotation_file_path = '/Users/cxs/Downloads/scored studies/00000818-113072/00000818-113072.rml'

edf_reader = PhilipsEDFReader(edf_file_path)
annotation_reader = PhilipsAnnotationReader(annotation_file_path)
print(annotation_reader.get_standard_AH_events())
print(edf_reader.signal_labels)
chest = edf_reader.get_signal(ch_name='Effort ABD', tmin=3600, tmax=3660)
abdomen = edf_reader.get_signal(ch_name='Effort THO', tmin=3600, tmax=3660)
ecg = edf_reader.get_signal(ch_name='ECG I', tmin=3600, tmax=3660)
ppg = edf_reader.get_signal(ch_name='Pleth', tmin=3600, tmax=3660)
bcg = edf_reader.get_signal(ch_name='electric_data', tmin=3600, tmax=3660)

fig, axes = plt.subplots(5, 1)
axes[0].plot(chest, label='chest')
axes[1].plot(abdomen, label='abdomen')
axes[2].plot(ecg, label='ecg')
axes[3].plot(ppg, label='ppg')
axes[4].plot(bcg, label='bcg')
plt.show()

AH_events = annotation_reader.get_standard_AH_events()
sleep_stages = annotation_reader.get_standard_sleep_stages()
print(AH_events)
print(sleep_stages)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-

"""
@File        :   EventDetector 
@Time        :   2023/9/25 17:14
@Author      :   Xuesong Chen
@Description :   
"""

import pandas as pd
import dcase_util
from wuji import sed_eval


def compute_metrics(ref: pd.DataFrame, pred: pd.DataFrame, print_logs=True, **kwargs):
    '''
    :param pred and ref:
    :return:
    '''
    pred_event_list = dcase_util.containers.MetaDataContainer(
        pred.to_dict(orient='records')
    )
    ref_event_list = dcase_util.containers.MetaDataContainer(
        ref.to_dict(orient='records')
    )

    # Default parameters for EventBasedMetrics
    default_params = {
        't_collar': 0.5,
        'evaluate_onset': True,
        'evaluate_offset': True,
        'percentage_of_length': 0.5,
            # offset可容忍的误差，即offset误差在percentage_of_length*duration内都算正确，越接近于1，越宽松
        'onset_offset_op': 'or'
    }
    default_params.update(kwargs)

    event_based_metrics = sed_eval.sound_event.EventBasedMetrics(
        event_label_list=ref_event_list.unique_event_labels,
        **default_params
    )

    for filename in ref_event_list.unique_files:
        reference_event_list_for_current_file = ref_event_list.filter(
            filename=filename
        )

        estimated_event_list_for_current_file = pred_event_list.filter(
            filename=filename
        )

        event_based_metrics.evaluate(
            reference_event_list=reference_event_list_for_current_file,
            estimated_event_list=estimated_event_list_for_current_file
        )

    if print_logs:
        print(event_based_metrics)
    else:
        return event_based_metrics


def convert_df_to_sed_type(df, file, scene_label):
    df['event_offset'] = df['Start'] + df['Duration']
    df.rename(columns={
        'Type': 'event_label',
        'Start': 'event_onset',
    }, inplace=True
    )
    df['file'] = file
    df['scene_label'] = scene_label
    df.drop(labels='Duration', axis=1, inplace=True)
    return df


class EventDetectorEvaluator:
    def __init__(self, gt_df, pred_df, file='pseudo_file', scene_label='pseudo_scene', **kwargs):
        gt_df = gt_df.copy()
        pred_df = pred_df.copy()
        self.gt_df = convert_df_to_sed_type(gt_df, file, scene_label)
        self.pred_df = convert_df_to_sed_type(pred_df, file, scene_label)
        self.metrics = compute_metrics(self.gt_df, self.pred_df, print_logs=False, **kwargs).results()

    def _get_metric(self, metric, label='overall', average='macro'):
        if label == 'overall':
            if average == 'macro':
                if metric in ['f_measure', 'precision', 'recall']:
                    if not self.metrics['class_wise_average']['f_measure']:
                        return 0
                    else:
                        return self.metrics['class_wise_average']['f_measure'][metric]
            elif average == 'micro':
                if metric in ['f_measure', 'precision', 'recall']:
                    return self.metrics['overall']['f_measure'][metric]
        else:
            if label not in self.metrics['class_wise']:
                return 0
            if metric in ['f_measure', 'precision', 'recall']:
                return self.metrics['class_wise'][label]['f_measure'][metric]

    def recall(self, label='overall', average='macro'):
        return self._get_metric('recall', label, average)

    def precision(self, label='overall', average='macro'):
        return self._get_metric('precision', label, average)

    def f1_score(self, label='overall', average='macro'):
        return self._get_metric('f_measure', label, average)
